wvdial = ('[Dialer Defaults]\n'
          'Init1 = ATZ\n'
          'Init2 = ATQ0\n'
          'Init3 = AT+CGDCONT=1,"IP","apn.konekt.io"\n'
          'Phone = *99***1#\n'
          'Stupid Mode = 1\n'
          'Username = { }\n'
          'Password = { }\n'
          'Modem Type = Analog Modem\n'
          'Modem = /dev/ttyUSB0\n'
          'IDSN = 0\n')
